<?php

/**
 * MVCnPHP - BaseView.class.php
 *
 * This source file is subject to version 2.02 of the PHP license, that is bundled with this 
 * package in the file LICENSE, and is available at through the world-wide-web at
 * http://www.php.net/license/2_02.txt. If you did not receive a copy of the PHP license and are 
 * unable to obtain it through the world-wide-web, please send a note to license@php.net so we can
 * mail you a copy immediately.
 *
 * @author Tony Bibbs <tony@geeklog.net>
 * @copyright Tony Bibbs 2003-2005
 * @package net.geeklog.mvc
 * @version $Id:$
 *
 */

/**
 * Defines the interface all views must implement
 */
require_once 'CommandInterface.php';

/**
* Abstract class for views
*
* @author Tony Bibbs <tony@geeklog.net>
* @package net.geeklog.mvc
*
*/
abstract class MVCnPHP_BaseCommand implements MVCnPHP_CommandInterface {
    /**
     * Indicates if a view requires it is run under SSL
     * @var boolean
     * @access private
     */
    private $requiresSSL = false;
    
    /**
     * Constructor
     *
     * Checks if SSL is required and if it is but fails the check it will throw an exception
     *
     * @author Tony Bibbs <tony@geeklog.net>
     * @access public
     * @throws Exception Throws an exception if the view fails the SSL check
     *
     */
    public function __construct()    
    {
        if ($this->requiresSSL) {
            if (!stristr($_ENV['SERVER_PROTOCOL'],'https')) {
                throw new Exception('View requires SSL');
            }
        }
    }
    
    /**
     * Sets whether or not SSL is required by the view
     * 
     * @author Tony Bibbs <tony@geeklog.net>
     * @access public
     * @param boolean $switch Indicator of whether or not SSL is required
     * 
     */
    public function setSSLRequired($switch)
    {
        $this->requiresSSL = $switch;
    }
    
    /**
     * Gets whether or not SSL is required
     *
     * @author Tony Bibbs <tony@geeklog.net>
     * @access public
     * @return boolean Whether or note the view requires SSL
     *
     */
    public function getSSLRequired()
    {
        return $this->requiresSSL;
    }
    
    /**
     * This short piece of code converts a request into a set of model objects that the 
     * developer can work directly with.
     *
     * It is possible to map form fields directly to model objects so that the command that catches
     * the request can begin working with the objects immediately and not have to do the form field
     * to domain object mapping themselves.  
     *
     * NOTE: should one of the setters cause an exception to get thrown, we simply ignore them and 
     * continue processing.  This is important because Propel's date/time setters() expect a 
     * format argument that we aren't giving them so we'll just ignore it and let the caller fo this
     * method try and figure out the problem.  For those of you not using Propel, simply note that
     * method will not die should it encounter an error calling the setter.
     *
     * To Illustrate, assume you have the following in an HTML form:
     * <code>
     * <form action="index.php" method="POST">
     *     <input type="hidden" name="cmd" value="saveCustomer"/>
     *     <input type="hidden" name="Customer->customerNumber"/>
     *     <input type="text" name="Customer->firstName"/>
     *     <input type="text" name="Customer->lastName"/>
     *     <input type="text" name="Customer->phoneNumber"/>
     * </form>
     * </code>
     *
     * The the command that represents saveCustomer would call this method.  This method assumes that
     * the Customer class was already included and that it has the methods setCustomerNumber(), 
     * setFirstName(), setLastName() and setPhoneNumber() defined.  If that is the case this method
     * would then return an array containig a singe Customer object that has be prefilled with the 
     * data submitted on the form.
     *
     * This is also as good a time as any to put a plug in for Propel, http://propel.phpdb.org.  If
     * you choose to use Propel, the power that this method provides is even more 
     * @author Tony Bibbs <tony@geeklog.net>
     * @access public
     * @param array $objArray You can pass in pre-existing propel objects
     * @return array Collection of domain objects
	 * @todo This function is currently unsophisticated.  More features may be added to this to do 
	 * things like allow associated one domain object as a property on another domain object.  
	 * Until we have a need I'll simply leave this method as is
     *
     */
    protected function requestToObjects($objInstances=array())
	{
	    if (!is_array($objInstances)) {
	        $objInstances = array();
	    }
	    
		// Gets the field names from the request
		$postVars = array_keys($_REQUEST);
		
		// Determine the name of the objects we need to create and the setters we'll need to call
		foreach ($postVars as $curVarName) {
			if (stristr($curVarName, '->')) {
				$tmpArray = explode('->',$curVarName);
				if (strstr($tmpArray[0],'_')) {
				    $nameOfClass = explode('_',$tmpArray[0]);
				    $nameOfClass = $nameOfClass[0];
				} else {
				    $nameOfClass = $tmpArray[0];
				}
				$objArray[$tmpArray[0]]['_NAME_OF_CLASS_'] = $nameOfClass;
				$objArray[$tmpArray[0]][$tmpArray[1]] = $_REQUEST[$curVarName];
			}
		}
		$objNames = @array_keys($objArray);
		
        // Now create each object and set the appropriate properties
        if(is_array($objNames)){
        	foreach ($objNames as $curObj) {		    
        		if (empty($objInstances[$curObj])) {
        		    $classToCreate = $objArray[$curObj]['_NAME_OF_CLASS_'];
        			//$obj = new $curObj();
        			$obj = new $classToCreate();
        			$objInstances[$curObj] = $obj;
        		} else {
        			$obj = $objInstances[$curObj];
        		}
        		$objAttributes = array_keys($objArray[$curObj]);
        		foreach ($objAttributes as $curAttribute) {
        		    if ($curAttribute <> '_NAME_OF_CLASS_') {
        		    	try {
            				$fnName = 'set' . ucwords($curAttribute);
            				$obj->$fnName($objArray[$curObj][$curAttribute]);
        		    	} catch(Exception $e) {
                            $_SESSION['requestToObjectsErrors'][] = $fnName.'('.$objArray[$curObj][$curAttribute].') Error: Invalid Data.';
        		    	}
        		    }
        		}
        		$objInstances[$curObj] = $obj;
        	}
        }
		
        // Return the object(s) that were created
        return $objInstances;
	}
}

?>