<?php

/* Reminder: always indent with 4 spaces (no tabs). */

/**
 * Department of Administrative Services
 * Information Technology Enterprise
 *
 * ITE PHP5 Enterprise Enterprise Authentication and Authorization (A&A) Client
 *
 * This source file is subject to version 2.02 of the PHP license, that is bundled with this 
 * package in the file LICENSE, and is available at through the world-wide-web at
 * http://www.php.net/license/2_02.txt. If you did not receive a copy of the PHP license and are 
 * unable to obtain it through the world-wide-web, please send a note to license@php.net so we can 
 * mail you a copy immediately.
 *
 * @author Tony Bibbs <tony.bibbs@iowa.gov>
 * @copyright 2005 State of Iowa
 * @version $Id:$
 *
 */

/**
 * The A&A service API
 */
require_once 'ITE/AAClient/ServiceInterface.php';

/**
 * This file defines the application program interface into ITE's Enterprise A&A Service
 *
 * @package gov.iowa.das.ite.aaclient
 *
 */
class ITE_AAClient_Client implements ITE_AAClient_ServiceInterface  {
    /**
     * Holds the configured A&A settings
     * @var array
     * @access private
     */
    private $options = array();
    
    /**
     * Holds instance of the handler we should use to talk to the A&A Service
     * @var string
     * @access private
     */
    private $handler = null;
        
    /**
     * Constructor
     *
     * Configure the A&A client so that all subsequent calls are more convenient.
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access public 
     * @param string $handler Method of communication to use, can use.  Only XML is supported right 
     * now.  SOAP support will be added.
     * @param string $appID Application ID that was assigned to the application from the A&A 
     * Service.  Customers must contact ITE to set-up application ID's for new A&A enabled
     * applications.
     * @param string $aaServer Host name of the A&A Server
     * @param int $aaPort Port to talk over on A&A Server
     * @param boolean $useSSL Whether or not we should talk over SSL to the A&A Service
     * @return void
     * @todo Eventually the A&A service will be expanded to support SOAP.  when that happens, this
     * method should still work fine, it's only that the handler given would change.  This is an 
     * FYI.
     *
     */
    public function __construct($handler, $appId, $aaServer, $aaPath = '/entaa/', $useSSL=true, 
        $aaPort='')
    {
        $this->options = array('appId'=>$appId,'aaServer'=>$aaServer,'aaPath'=>$aaPath,
            'useSSL'=>$useSSL, 'aaPort'=>$aaPort);
        // Instantiate the right handler.
        $handlerFileName = sprintf('%sHandler.php', $handler);
        require_once $handlerFileName;
        $handlerClass = sprintf('ITE_AAClient_%sHandler', $handler);        
        $this->handler = new $handlerClass($appId, $aaServer, $aaPath, $useSSL, $aaPort);
         
    }
    
    /**
     * Authenticates a user
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access public
     * @param string $userId ID for the user logging in
     * @param string $password Password to authenticate user with
     * @return ITE_AAClient_User A&A user object that is serializable
     *
     */
    public function &authenticate($userId, $password, $providerId = '')
    {
        $userObj = &$this->handler->authenticate($userId, $password, $providerId);
        
        // That's right, we are sending this object as part of the user object.  All methods on the
        // user object simply proxy back here and we proxy on to the handler.
        $userObj->setHandler($this);
        
        return $userObj;
    }
    
    /**
     * Creates a new Enterprise A&A account in the repository tied to the application
     *
     * NOTE: this method does not work yet on the Active Directory provider.
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access public
     * @param string $userId ID to assign to the new user
     * @param string $password Initial password for the user
     * @param string $fname User's first name
     * @param string $lname User's last name
     * @param string $email User's email address
     * @param array $privArray Array of privileges to assign to the user
     *
     */
    public function createAccount($userId, $password, $fname, $lname, $email = '', $privArray = '')
    {
        if (empty($privArray)) {
            $privArray = array();
        }
        $this->handler->createAccount($userId, $password, $fname, $lname, $email = '', 
            $privArray = '');
    }
    
    /**
     * Gets the user object from the A&A service tied to the given SSO token
     *
     * Enterprise A&A does support Single Sign-on. When an application uses SSO, we take them to 
     * the A&A SSO login page and when they successfully login they are returned to the calling
     * application and are passed the user's SSO token.  From there the calling application would
     * use this method to get the user object for the authenticated user
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access pulbic
     * @param string $ssoToken The SSO token tied to the user
     * @return ITE_AAClient_User A&A User object
     *
     */
    public function &getUserObject($ssoToken)
    {
        if (empty($ssoToken)) {
            require_once 'PEAR/Exception.php';
            throw new PEAR_Exception('You must give a valid SSO token');
        }
        
        $userObj = &$this->handler->getUserObject($ssoToken);
        
        // That's right, we are sending this object as part of the user object.  All methods on the
        // user object simply proxy back here and we proxy on to the handler.
        $userObj->setHandler($this);
        
        return $userObj;
    }
    
    /**
     * Get login URL
     *
     * Returns the Enterprise A&A SSO Login url that can be used by calling applications to
     * redirect users to the SSO login page when needed
     *
     * @author Tony Bibbbs <tony.bibbs@iowa.gov>
     * @access public
     * @param string $returnUrl URL to send the users to once they successfully login
     * @return string The URL to the A&A SSO login page
     *
     */
    public function getSignOnUrl($returnUrl)
    {
        if ($this->options['useSSL']) {
            $proto = 'https:';
        } else {
            $proto = 'http:';
        }
        return sprintf('%s://%s/%ssso?appId=%s&callingApp=%s', $proto, $this->options['aaServer'],
            $this->options['aaPath'],$this->options['appId'], urlencode($returnUrl));        
    }
    
    /**
     * Get signoff URL
     *
     * Returns the Enterprise A&A SSO logout URL that can be used by calling applications to kill
     * a SSO session
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access public 
     * @param string $returnUrl URL to send the users to once they successfully logout
     * @return string The URL to the A&A SSO logout page
     *
     */
    function getSignOffUrl($returnUrl)
    {
        if ($this->options['useSSL']) {
            $proto = 'https';
        } else {
            $proto = 'http';
        }
        return sprintf('%s://%s%ssso?commandName=Logoff&appId=%s&callingApp=%s', $proto, 
            $this->options['aaServer'], $this->options['aaPath'],$this->options['appId'], 
            urlencode($returnUrl));        
    }
}

?>