<?php

/* Reminder: always indent with 4 spaces (no tabs). */

/**
* The parent class that already has a handle to an instantiated flexy object
*/
require_once 'BaseViewFlexy.php';

/**
* The propel user model class
*/
require_once 'PHPARCH/Account.php';

/**
* Based on the flexy object but gets a handle to a user object out of the
* session if one is available.
*
* NOTE: it is assumed that descendants of this view class need to
* ensure the user is logged in.  To do so, the login check happens
* in the getView() method which is invoked by the controller.  It then
* calls processView() on the child.  Descendants, therefore, must
* implement processView() and NOT getView()
*
* @author Tony Bibbs <tony@geeklog.net>
*
*/
abstract class PHPARCH_BaseViewFlexyUser extends PHPARCH_BaseViewFlexy {
    /**
     * User Object
     * @access public
     * @var object
     */
    public $user = null;
    
    /**
     * Indicates if the user object is require (i.e. user must have logged in)
     * @access private
     * @var boolean
     */
    private $userRequired = true;
    
    /**
     * Holds a set of privileges that a user must have one of in order to be able to see this view.
     * @access private
     * @var array
     */
    private $privilegesToView = array();
    
    /**
    * Initializes the user object so it is available to children
    *
    * @author Tony Bibbs <tony@geeklog.net>
    * @access public
    * 
    */
    public function __construct()
    {
        global $conf;
        
        // Try to get the user object out of the session.
    	$this->user = unserialize($_SESSION['user']);
        
    	// Determine if the view requires the user object or if it is simply optional.  NOTE the 
    	// user object is required by default
        if ($this->getUserRequired() AND !is_object($this->user)) {
            // NOTE this makes use of the global forward feature in MVCnPHP
            $_SESSION['postLoginURL'] = $conf['site_url'] . '/' . $_SERVER['REQUEST_URI'];
            
            throw new Exception('doForward:login');
        }

        // Ensure user can even see this view
        if (!$this->authorizedToView()) {
            throw new Exception('You do not have sufficient privileges to view this page.  Please
                note that all attempts to illegally access a page are logged');
        }
    }
    
    /**
    * Starts the processing of the view
    *
    * This will check to ensure the user session is valid. If not it will forward
    * them on to the login page otherwise it calls _processView() on the child
    *
    * @author Tony Bibbs <tony@geeklog.net>
    * @access public
    * @return string Forward from descendant if given one
    *
    */
    public function getView()
    {
    	$this->initializeFlexy();
    	     
        return $this->processView();
    }
    
    /**
    * Determines if we have a valid user object.  Used by Flexy
    *
    * @author Tony Bibbs <tony@geeklog.net>
    * @access public
    * @return boolean True if we have a user object, otherwise false
    *
    */
    public function haveUser()
    {
        return is_object($this->user);
    }

	/**
	* Determines if a number is odd.
	*
	* This is used by the Flexy template to
	* shade every other row for better readability
	*
	* @author Tony Bibbs <tony@geeklog.net>
	* @access public
	* @param int $key A whole number
	* @return int 1 if number is odd, otherwise 0
	*
	*/
	public function isOdd($key) 
	{
	    return $key % 2;	   
	}
	
	/**
	* Returns the username for the current user session we are working with
	*
	* @author Tony Bibbs <tony@geeklog.net>
	* @access public
	* @return string Username
	*
	*/
	public function getUserName()
	{
		return $this->user->getUserName();
	}
	
	/**
	* Returns the user ID fo the current user session we are working with
	*
	* @author Tony Bibbs <tony@geeklog.net>
	* @access public
	* @return int User ID
	*
	*/
	public function getUserId()
	{
		return $this->user->getUserId();
	}
	
	/**
	 * Sets whether or not the view requires the user object to be valid
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access protected
	 * @param boolean $string Whether or not the view requires a logged in user
	 *
	 */
	protected function setUserRequired($switch)
	{
		$this->userRequired = $switch;
	}
	
	/**
	 * Gets whether or not the view requires the user object to be valid
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access protected
	 * @return boolean Whether or not the view requires a logged in user
	 *
	 */
	protected function getUserRequired()
	{
		return $this->userRequired;
	}
	
	/**
	 * Allows views to register one or more privileges that would be required in order to even view
	 * a screen. All privileges will be compared in an OR relationship.  If you need more 
	 * specialized handling then you should override this method.
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access protected
	 * @param array $privileges Array of privleges
	 *
	 */
	protected function setPrivilegesToView($privileges = array())
	{
	    $this->privilegesToView = $privileges;
	}
	
	/**
	 * Provides a slick method for authorizing a user has sufficient privileges to view a screen
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access protected
	 * @return boolean True if the user can view the screen, otherwise false
	 *
	 */
	protected function authorizedToView()
	{
	    if (count($this->privilegesToView) == 0) return true;
	    
	    foreach ($this->privilegesToView as $curPriv) {
	        if ($this->user->hasPrivilege($curPriv)) {
	            return true;
	        }
	    }
	    return false;
	}
	
	/**
	 * Function used from within Flexy templates to show specialized features to users.  
	 * 
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access public
	 * @param string $privilegeName Privilege name
	 * @return boolean True if they have the privilege, otherwise false
	 * @see PHPARCH_BaseViewFlexy::hasPrivilege
	 *
	 */
	public function hasPrivilege($privilegeName)
	{
	    if (!is_object($this->user)) return false;
	    throw new Exception();
	    return $this->user->hasPrivilege($privilegeName);
	}
}